﻿using Grasshopper.Kernel;
using Grasshopper.Kernel.Types;

namespace CustomTypeExample
{
    public class TriStateType : GH_Goo<int>
    {
        public TriStateType()
        {
            this.Value = -1;
        }

        public TriStateType(int tristateValue)
        {
            this.Value = tristateValue;
        }

        public TriStateType(TriStateType tristateSource)
        {
            this.Value = tristateSource.Value;
        }

        public override IGH_Goo Duplicate()
        {
            return new TriStateType(this);
        }
        public override int Value
        {
            get { return base.Value; }
            set
            {
                if (value < -1) { value = -1; }
                if (value > +1) { value = +1; }
                base.Value = value;
            }
        }
        public override bool IsValid
        {
            get { return true; }
        }

        public override string TypeName
        {
            get { return "TriState"; }
        }

        public override string TypeDescription
        {
            get { return "A TriState Value (True, False or Unknown)"; }
        }

        public override string ToString()
        {
            if (this.Value == 0) { return "False"; }
            if (this.Value > 0) { return "True"; }
            return "Unknown";
        }

        public override bool Write(GH_IO.Serialization.GH_IWriter writer)
        {
            writer.SetInt32("tri", this.Value);
            return true;
        }

        public override bool Read(GH_IO.Serialization.GH_IReader reader)
        {
            this.Value = reader.GetInt32("tri");
            return true;
        }
        public override object ScriptVariable()
        {
            return this.Value;
        }

        public override bool CastTo<Q>(ref Q target)
        {
            if (typeof(Q).IsAssignableFrom(typeof(int)))
            {
                object ptr = this.Value;
                target = (Q)ptr;
                return true;
            }

            if (typeof(Q).IsAssignableFrom(typeof(GH_Integer)))
            {
                object ptr = new GH_Integer(this.Value);
                target = (Q)ptr;
                return true;
            }
            return false;
        }
        public override bool CastFrom(object source)
        {
            if (source == null) { return false; }
            int val;
            if (GH_Convert.ToInt32(source, out val, GH_Conversion.Both))
            {
                this.Value = val;
                return true;
            }
            string str = null;
            if (GH_Convert.ToString(source, out str, GH_Conversion.Both))
            {
                switch (str.ToUpperInvariant())
                {
                    case "FALSE":
                    case "F":
                    case "NO":
                    case "N":
                        this.Value = 0;
                        return true;

                    case "TRUE":
                    case "T":
                    case "YES":
                    case "Y":
                        this.Value = +1;
                        return true;

                    case "UNKNOWN":
                    case "UNSET":
                    case "MAYBE":
                    case "DUNNO":
                    case "?":
                        this.Value = -1;
                        return true;
                }
            }
            return false;
        }
    }
}
